#!D:\bld\xfab_1644315860030\_h_env\python.exe

# Create objects to manipulate - they hold your data
#
from __future__ import absolute_import
import sys
from ImageD11 import peakmerge, indexing, transformer
mypeakmerger = peakmerge.peakmerger()
mytransformer = transformer.transformer()
myindexer = indexing.indexer()
#
# Your work starts here:
#
filename = sys.argv[1]
outfile =  sys.argv[2]
parfile =  sys.argv[3]
mytransformer.loadfiltered(  filename )
mytransformer.loadfileparameters( parfile )
mytransformer.compute_tth_eta( )
mytransformer.getcolumn(  'tth' )
mytransformer.getcolumn(  'eta' )
mytransformer.addcellpeaks( )
mytransformer.computegv( )
mytransformer.savegv(  outfile )
